local ReportAfter = 120;
local CheckFrequency = 30;
local PlayerCoords = {};
local NextAFKCheck = 0;
local Config;
local EnabledFrame = CreateFrame("Frame");

local CheckButtons = {
  {Text="Enable",SavedVar="Enabled",Default=true,
    TooltipText="Enables or disables the reporting of AFK players.",
    Func = function(Value)
      if Value then
        EnabledFrame:Show();
      else
        EnabledFrame:Hide();
      end
    end},
  {Text="Play Sound",SavedVar="PlaySound",Default=true,
    TooltipText="Plays a sound when reporting a player for being AFK."},
};


local EventFuncs = {
  VARIABLES_LOADED = function(self)
    Config = BaudAFKReporterConfig;
    if not Config then
      Config = {};
      BaudAFKReporterConfig = Config;
      Config.PlaySound = BaudAFKReporterSound; --Backwards compatability
    end
    for Key, Value in ipairs(CheckButtons)do
      if(Config[Value.SavedVar]==nil)then
        Config[Value.SavedVar] = Value.Default;
      end
      Value.Button:SetChecked(Config[Value.SavedVar]);
      if Value.Func then
        Value.Func(Config[Value.SavedVar]);
      end
    end
  end,

  PLAYER_ENTERING_WORLD = function(self)
    local InInstance, InstanceType = IsInInstance();
    if(InstanceType~="pvp")or(GetBattlefieldInstanceExpiration() > 0)then
      self:Hide();
      return;
    end
    wipe(PlayerCoords);
    self:Show();
    SetMapToCurrentZone();
  end,

  UPDATE_BATTLEFIELD_STATUS = function(self)
    if(GetBattlefieldInstanceExpiration() > 0)then
      self:Hide();
    end
  end,
};


function BaudAFKReporter_OnLoad(self)
  self:SetParent(EnabledFrame);

  for Key, Value in pairs(EventFuncs)do
    self:RegisterEvent(Key);
  end
  self:SetScript("OnEvent", function(self, event)
    EventFuncs[event](self);
  end);
  self:SetScript("OnUpdate", BaudAFKReporter_OnUpdate);

  local Frame = CreateFrame("Frame", "BaudAFKReporterOptions");
  local Text = Frame:CreateFontString(nil, "BACKGROUND", "GameFontHighlight");
  Text:SetText("Baud AFK Reporter uses the world map to track other players of the same faction. Players who do not move "
    .."for two minutes or more will be reported for being AFK.");
  Text:SetJustifyH("LEFT");
  Text:SetJustifyV("TOP");
  Text:SetPoint("TOPLEFT", 20, -20);
  Text:SetPoint("BOTTOMRIGHT", -20, 0);
  local Button;
  for Key, Value in ipairs(CheckButtons)do
    Button = CreateFrame("CheckButton", "BaudAFKReporterCheck"..Key, Frame, "OptionsCheckButtonTemplate");
    Button:SetPoint("TOPLEFT", 20, -40 - 25 * Key);
    Button.tooltipText = Value.TooltipText;
    getglobal(Button:GetName().."Text"):SetText(Value.Text);
    Button:SetScript("OnClick", BaudAFKReporterCheck_OnClick);
    Button.Info = Value;
    Value.Button = Button;
  end

  Text = Frame:CreateFontString(nil, "BACKGROUND", "GameFontNormalSmall");
  Text:SetPoint("BOTTOMRIGHT",-13,13);
  Text:SetText("Version "..GetAddOnMetadata("BaudAFKReporter","Version"));

  Frame.name = "Baud AFK Reporter";
  InterfaceOptions_AddCategory(Frame);


  SlashCmdList["BaudAFKReporter"] = function()
    InterfaceOptionsFrame_OpenToCategory("Baud AFK Reporter");
  end
  SLASH_BaudAFKReporter1 = "/baudafk";
  SLASH_BaudAFKReporter2 = "/baudafkreporter";

  --DEFAULT_CHAT_FRAME:AddMessage("Baud AFK Reporter: AddOn Loaded.");
end


function BaudAFKReporterCheck_OnClick(self)
  Config[self.Info.SavedVar] = (self:GetChecked()==1);
  if self.Info.Func then
    self.Info.Func(Config[self.Info.SavedVar]);
  end
  if self:GetChecked()then
    PlaySound("igMainMenuOptionCheckBoxOff");
  else
    PlaySound("igMainMenuOptionCheckBoxOn");
  end
end


function BaudAFKReporter_OnUpdate()
  if(NextAFKCheck > GetTime())then
    return;
  end
  NextAFKCheck = GetTime() + CheckFrequency;
  local Name, Server, Key, X, Y, Unit, PlayedSound;
  if(GetNumRaidMembers() > 0)then
    for Index = 1, MAX_RAID_MEMBERS do
      Unit = "raid"..Index;
      Name, Server = UnitName(Unit);
      if Name and not UnitIsUnit(Unit, "player")then
        Key = Name;
        if Server and(Server~="")then
          Key = Key.."-"..Server;
        end
        X, Y = GetPlayerMapPosition(Unit);
        if(X==0)and(Y==0)then
          if PlayerCoords[Key]then
            PlayerCoords[Key] = nil;
          end
        else
          if PlayerCoords[Key]then
            if(PlayerCoords[Key].X==X)and(PlayerCoords[Key].Y==Y)then
              if(GetTime() - PlayerCoords[Key].Start >= ReportAfter)and not PlayerCoords[Key].Reported then
                DEFAULT_CHAT_FRAME:AddMessage("Reporting AFK player: "..Key, 1, 0, 1);
                ReportPlayerIsPVPAFK(Unit);
                PlayerCoords[Key].Reported = true;
                if not PlayedSound and(Config.PlaySound~=false)then
                  PlaySoundFile("Sound\\Creature\\MaidenOfVirtue\\MaidenSlay03.wav");
                  PlayedSound = true;
                end
              end
            else
              PlayerCoords[Key].X = X;
              PlayerCoords[Key].Y = Y;
              PlayerCoords[Key].Start = GetTime();
              PlayerCoords[Key].Reported = nil;
            end
          else
            PlayerCoords[Key] = {Start = GetTime(), X = X, Y = Y};
          end
        end
      end
    end
  end
end