local Fonter = LibStub("AceAddon-3.0"):NewAddon("Fonter", "AceConsole-3.0", "AceEvent-3.0")
local LSM    = LibStub:GetLibrary("LibSharedMedia-3.0")

function Fonter:OnInitialize()
	self:InitializeConfiguration()
	self:InitializeConfigurationGUI()
	self:InitializeFonts()
	self:RegisterEvent("VARIABLES_LOADED", "OnVariablesLoaded")
end

function Fonter:OnVariablesLoaded()
end

function Fonter:InitializeFonts()
	LSM.RegisterCallback(Fonter, "LibSharedMedia_Registered", "LibSharedMediaCallback")
	LSM.RegisterCallback(Fonter, "LibSharedMedia_SetGlobal" , "LibSharedMediaCallback")

	-- Fonts (http://www.1001freefonts.com/)
	self.Fonts = {
["Accidental Presidency"] = 
"Interface\\Addons\\Fonter\\Fonts\\AccidentalP.ttf",
["Chumbly"] = 
"Interface\\Addons\\Fonter\\Fonts\\Chumbly.ttf",
["Chen"] = 
"Interface\\Addons\\Fonter\\Fonts\\Chen.ttf",
["Collegia"] = 
"Interface\\Addons\\Fonter\\Fonts\\Collegia.ttf",
["Excite"] = 
"Interface\\Addons\\Fonter\\Fonts\\Excite.ttf",
["Gunplay"] = 
"Interface\\Addons\\Fonter\\Fonts\\Gunplay.ttf",
["Hemi Head"] = 
"Interface\\Addons\\Fonter\\Fonts\\HemiHead.ttf",
["Jekyl"] =
"Interface\\Addons\\Fonter\\Fonts\\Jekyl.ttf",
["Lane Humour"] = 
"Interface\\Addons\\Fonter\\Fonts\\LaneHumour.ttf",
["Lexi"] = 
"Interface\\Addons\\Fonter\\Fonts\\Lexi.ttf",
["Monoglyceride"] = 
"Interface\\Addons\\Fonter\\Fonts\\Monoglyceride.ttf",
["Vinque"] = 
"Interface\\Addons\\Fonter\\Fonts\\Vinque.ttf",
	}

	for k, v in pairs(self.Fonts) do
		LSM:Register("font", k, v)
	end
end

function Fonter:LibSharedMediaCallback(Name, MediaType, Key)
	if MediaType == "font" then
		self:Update()
	end
end

function Fonter:Update()
	self:SetBaseFonts()
end

function Fonter:SetFont(Object, Font, Size, Style)
	Object:SetFont(Font, Size, Style)
end

function Fonter:SetBaseFonts()
	local FontCenter    = LSM:Fetch("font", self.db.profile.FontCenter)
	local FontChat      = LSM:Fetch("font", self.db.profile.FontChat)
	local FontDamage    = LSM:Fetch("font", self.db.profile.FontDamage)
	local FontHeader    = LSM:Fetch("font", self.db.profile.FontHeader)
	local FontStacks    = LSM:Fetch("font", self.db.profile.FontStacks)
	local FontStandard  = LSM:Fetch("font", self.db.profile.FontStandard)
	local FontNamePlate = LSM:Fetch("font", self.db.profile.FontNamePlate)
	local FontUnit      = LSM:Fetch("font", self.db.profile.FontUnit)
	local FontSmall     = FontStandard

	local SizeSmall  = self.db.profile.SizeSmall
	local SizeMedium = self.db.profile.SizeMedium
	local SizeLarge  = self.db.profile.SizeLarge
	local SizeHuge   = self.db.profile.SizeHuge
	local SizeInsane = self.db.profile.SizeHuge + 8

	self:SetFont(SystemFont_OutlineThick_Huge2     , FontCenter, SizeHuge, "THICKOUTLINE")
	self:SetFont(SystemFont_Shadow_Outline_Huge2   , FontCenter, SizeHuge, "THICKOUTLINE")
	self:SetFont(SystemFont_OutlineThick_WTF       , FontCenter, SizeInsane, "THICKOUTLINE")
	self:SetFont(SystemFont_OutlineThick_Huge4     , FontCenter, SizeHuge, "THICKOUTLINE")
	self:SetFont(SystemFont_Shadow_Huge1           , FontCenter, SizeHuge, nil)
	self:SetFont(NumberFont_Shadow_Med             , FontChat, SizeMedium, nil)
	self:SetFont(SystemFont_Shadow_Huge3           , FontDamage, SizeHuge, nil)
	self:SetFont(QuestFont_Large                   , FontHeader, SizeMedium, nil)
	self:SetFont(QuestFont_Shadow_Huge             , FontHeader, SizeHuge, nil)
	self:SetFont(MailFont_Large                    , FontHeader, SizeMedium, nil)
	self:SetFont(SystemFont_Med1                   , FontNamePlate, SizeMedium, nil)
	self:SetFont(SystemFont_Shadow_Med1            , FontNamePlate, SizeMedium, nil)
	self:SetFont(SystemFont_Med2                   , FontNamePlate, SizeMedium, nil)
	self:SetFont(SystemFont_Med3                   , FontNamePlate, SizeMedium, nil)
	self:SetFont(SystemFont_Shadow_Med3            , FontNamePlate, SizeMedium, nil)
	self:SetFont(NumberFont_OutlineThick_Mono_Small, FontNamePlate, SizeMedium, "OUTLINE")
	self:SetFont(InvoiceFont_Med                   , FontNamePlate, SizeMedium, nil)
	self:SetFont(FriendsFont_Normal                , FontNamePlate, SizeMedium, nil)
	self:SetFont(FriendsFont_Large                 , FontNamePlate, SizeLarge, nil)
	self:SetFont(SystemFont_Shadow_Small           , FontNamePlate, SizeSmall, nil)
	self:SetFont(NumberFont_Outline_Med            , FontStacks, SizeMedium, "OUTLINE")
	self:SetFont(GameTooltipHeader                 , FontStacks, SizeMedium, nil)
	self:SetFont(Tooltip_Small                     , FontStacks, SizeMedium, nil)
	self:SetFont(Tooltip_Med                       , FontStacks, SizeMedium, nil)
	self:SetFont(NumberFont_Outline_Large          , FontStacks, SizeLarge, "OUTLINE")
	self:SetFont(NumberFont_Outline_Huge           , FontStandard, SizeHuge, "OUTLINE")
	self:SetFont(SystemFont_Large                  , FontStandard, SizeLarge, nil)
	self:SetFont(SystemFont_Shadow_Large           , FontStandard, SizeLarge, nil)
	self:SetFont(SystemFont_Tiny                   , FontSmall, SizeSmall, nil)
	self:SetFont(SystemFont_Small                  , FontSmall, SizeSmall, nil)
	self:SetFont(SystemFont_Outline_Small          , FontSmall, SizeSmall, "OUTLINE")
	self:SetFont(SystemFont_InverseShadow_Small    , FontSmall, SizeSmall, nil)
	self:SetFont(NumberFont_Shadow_Small           , FontSmall, SizeSmall, nil)
	self:SetFont(SpellFont_Small                   , FontSmall, SizeSmall, nil)
	self:SetFont(InvoiceFont_Small                 , FontSmall, SizeSmall, nil)
	self:SetFont(AchievementFont_Small             , FontSmall, SizeSmall, nil)
	self:SetFont(ReputationDetailFont              , FontSmall, SizeSmall, nil)
	self:SetFont(FriendsFont_UserText              , FontSmall, SizeSmall, nil)
	self:SetFont(FriendsFont_Small                 , FontSmall, SizeSmall, nil)

	STANDARD_TEXT_FONT = FontStandard
	UNIT_NAME_FONT     = FontUnit
	DAMAGE_TEXT_FONT   = FontDamage
	NAMEPLATE_FONT     = FontNamePlate
	CHAT_FONT_HEIGHTS  = { 8, 10, 12, 14, 16, 18, 20, 22 }

		for i = 1, 7 do
			local Frame = _G["ChatFrame"..i]
			local Font, Size = Frame:GetFont()
			Frame:SetFont(FontChat, Size)
	end
end