local Fonter = LibStub("AceAddon-3.0"):GetAddon("Fonter")
local LSM    = LibStub:GetLibrary("LibSharedMedia-3.0")

function Fonter:InitializeConfiguration()
	-- Defaults
	self.ConfigurationDefaults	= {
		global = {
			Version		= 0,
		},
		profile = {
		  FontCenter		= "Gunplay",
		  FontChat			= "Accidental Presidency",
		  FontDamage		= "Hemi Head",
		  FontHeader		= "Lexi",
		  FontNamePlate		= "Chen",
		  FontStacks		= "Collegia",
		  FontStandard		= "Excite",
		  FontUnit			= "Vinque",
		  SizeSmall		= 10,
		  SizeMedium		= 12,
		  SizeLarge		= 14,
		  SizeHuge			= 16
		}
	}

	self.db = LibStub("AceDB-3.0"):New("FonterDB", self.ConfigurationDefaults, "Default")

	self.Configuration = { }

	self.Configuration.Main = { name = "Fonter Super", handler = Fonter, type = "group", get = "GetValue", set = "SetValue",
		args = {
			FontDefinitions = { name = "Font Definitions And Font Sizes", type = "group", inline = true, order = 1.0,
				args = {
					FontCenter = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontCenter
						end,
						set = function(info, v)
						  self.db.profile.FontCenter =  v
						  self:Update()
						end,
						name   = "Center Screen Font",
						desc   = "The font used for the Center Screen Text and Raid Warnings.",
						type   = "select",
						order  = 1.0
					},
					FontChat = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontChat
						end,
						set = function(info, v)
						  self.db.profile.FontChat =  v
						  self:Update()
						end,
						name   = "Chat/Chat Input",
						desc   = "The font used for the Chat Frame and Chat Input.",
						type   = "select",
						order  = 1.1
					},
					FontDamage = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontDamage
						end,
						set = function(info, v)
						  self.db.profile.FontDamage =  v
						  self:Update()
						end,
						name   = "Damage/Healing",
						desc   = "The font used for Damage/Healing Text including Scrolling Text and Experience.",
						type   = "select",
						order  = 1.2
					},
					FontHeader = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontHeader
						end,
						set = function(info, v)
						  self.db.profile.FontHeader =  v
						  self:Update()
						end,
						name   = "Headers",
						desc   = "The font used for headers in the Quest log, Books and Mail.",
						type   = "select",
						order  = 1.3
					},
					FontNamePlate = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontNamePlate
						end,
						set = function(info, v)
						  self.db.profile.FontNamePlate =  v
						  self:Update()
						end,
						name   = "Nameplates/Menus/Quest Log",
						desc   = "The font used for Nameplates, Menus and Quest Log.",
						type   = "select",
						order  = 1.4
					},
					FontStacks = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontStacks
						end,
						set = function(info, v)
						  self.db.profile.FontStacks =  v
						  self:Update()
						end,
						name   = "Stacks/Bag Frame/Tooltips",
						desc   = "The font used for Stacks, The Bag Frame and Tooltips.",
						type   = "select",
						order  = 1.5
					},
					FontStandard = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontStandard
						end,
						set = function(info, v)
						  self.db.profile.FontStandard =  v
						  self:Update()
						end,
						name   = "Standard/Undefined",
						desc   = "The font used for everything not specified by the other fonts.",
						type   = "select",
						order  = 1.6
					},
					FontUnit = {
						dialogControl = 'LSM30_Font',
						values = LSM:HashTable("font"),
						get = function() return self.db.profile.FontUnit
						end,
						set = function(info, v)
						  self.db.profile.FontUnit =  v
						  self:Update()
						end,
						name   = "Unit Names",
						desc   = "The font used for Unit Titles and Names (Above thier head).",
						type   = "select",
						order  = 1.7
					},
					SizeSmall = {
						name   = "Small",
						desc   = "Small text size.",
						type   = "range",
						step   = 1,
						min    = 6,
						max    = 16,
						order  = 1.8
					},
					SizeMedium = {
						name   = "Medium",
						desc   = "Medium text size.",
						type   = "range",
						step   = 1,
						min    = 8,
						max    = 18,
						order  = 1.9
					},
					SizeLarge = {
						name   = "Large",
						desc   = "Large text size.",
						type   = "range",
						step   = 1,
						min    = 10,
						max    = 20,
						order  = 2.0
					},
					SizeHuge = {
						name   = "Huge",
						desc   = "Huge text size.",
						type   = "range",
						step   = 1,
						min    = 12,
						max    = 22,
						order  = 2.1
					}
				}
			}
		}
	}

	self.Configuration.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.Configuration.Profiles.name = "Profiles"

	LibStub("AceConfig-3.0"):RegisterOptionsTable("Fonter",          self.Configuration.Main)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Fonter.Profiles", self.Configuration.Profiles)

	self.db.RegisterCallback(self, "OnProfileChanged", "Update")
	self.db.RegisterCallback(self, "OnProfileCopied",  "Update")
	self.db.RegisterCallback(self, "OnProfileReset",   "Update")

	self:RegisterChatCommand("Fonter", "ChatCommand")
	self:ValidateConfiguration()
end

function Fonter:ValidateConfiguration()
	-- Check for old or unknown configurations and reset
	if self.db.global.Version == 0 then
		self.db:ResetDB("Default")
		self:Print("Initializing configuration. Loading default values.")
		self.db.global.Version = 1
	end
end

function Fonter:InitializeConfigurationGUI()
	self.ConfigurationFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Fonter", self.Configuration.Main.name)

	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Fonter.Profiles", self.Configuration.Profiles.name, "Fonter Super")

	self.ConfigurationFrame.okay   = function() self:OnCloseConfigurationFrame() end
	self.ConfigurationFrame.cancel = function() self:OnCloseConfigurationFrame() end
end

function Fonter:ChatCommand()
	InterfaceOptionsFrame_OpenToCategory(self.ConfigurationFrame)
end

function Fonter:GetValue(Infotable)
	return self.db.profile[Infotable[#Infotable]]
end

function Fonter:SetValue(Infotable, Value)
	self.db.profile[Infotable[#Infotable]] = Value
	self:Update()
end

function Fonter:GetFontValue(Infotable)
	local table = LSM:List("font")

	for k, v in pairs(table) do
		if self.db.profile[Infotable[#Infotable]] == v then
			return k
		end
	end
end

function Fonter:SetFontValue(Infotable, Value)
	local table = LSM:List("font")
	self.db.profile[Infotable[#Infotable]] = table[Value]
	self:Update()
end